-- Please consult and update the ADRs in this repo when modifying this schema

-- NOTE This SQL is for initial setup of the database
-- It should not be run if the database already exists

--
-- Name: rbac; Type: SCHEMA; Schema: -; Owner: cc_rbac_api
--

CREATE USER cc_rbac;

CREATE SCHEMA rbac;

ALTER SCHEMA rbac OWNER TO cc_rbac;

CREATE USER cc_rbac_api;
GRANT USAGE ON SCHEMA rbac TO cc_rbac_api;
-- Don't grant DELETE, because we only allow soft deletes
GRANT SELECT, INSERT, UPDATE ON ALL TABLES IN SCHEMA rbac TO cc_rbac_api;
-- This lets this user advance with nextval (for inserts in tables using this) but not reset it
GRANT USAGE ON ALL SEQUENCES IN SCHEMA rbac TO cc_rbac_api;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT SELECT, INSERT, UPDATE ON TABLES TO cc_rbac_api;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT USAGE ON SEQUENCES TO cc_rbac_api;

CREATE USER cc_rbac_ro;
GRANT USAGE ON SCHEMA rbac TO cc_rbac_ro;
GRANT SELECT ON ALL TABLES IN SCHEMA rbac TO cc_rbac_ro;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT SELECT ON TABLES TO cc_rbac_ro;

--
-- Name: role_binding; Type: TABLE; Schema: rbac; Owner: cc_rbac
--

CREATE TABLE rbac.role_binding (
    id text PRIMARY KEY,
    user_id text NOT NULL,
    role_name text NOT NULL,
    organization_id text,
    account_id text,
    cloud_cluster_id text,
    logical_cluster_id text,
    cluster_type text,
    resource_type text,
    resource_name text,
    pattern_type text,
    deleted boolean DEFAULT false NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    created_by text NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    modified_by text NOT NULL,
    last_change_id bigint NOT NULL default 1
);

ALTER TABLE rbac.role_binding OWNER TO cc_rbac;

CREATE UNIQUE INDEX IF NOT EXISTS role_binding_last_change_id_idx on rbac.role_binding (last_change_id);

CREATE INDEX IF NOT EXISTS role_binding_scope_idx ON rbac.role_binding (organization_id, account_id, cloud_cluster_id, logical_cluster_id)
    WHERE deleted = false;
CREATE INDEX IF NOT EXISTS role_binding_user_idx ON rbac.role_binding (user_id)
    INCLUDE(deleted);
CREATE UNIQUE INDEX IF NOT EXISTS role_binding_unique_idx ON rbac.role_binding (
    user_id,
    role_name,
    COALESCE(organization_id, ''),
    COALESCE(account_id, ''),
    COALESCE(cloud_cluster_id, ''),
    COALESCE(logical_cluster_id, ''),
    COALESCE(cluster_type, ''),
    COALESCE(resource_type, ''),
    COALESCE(resource_name, ''),
    COALESCE(pattern_type, ''))
    WHERE deleted = false;
--
-- Name: role_binding_id_seq; Type: SEQUENCE; Schema: rbac; Owner: cc_rbac
--

-- This sequence is used by the hashid generator that supplies the id
CREATE SEQUENCE rbac.role_binding_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER TABLE rbac.role_binding_id_seq OWNER TO cc_rbac;

-- This sequence is used to generate a monotonically increasing value that can
-- be used by a Change Data Capture (CDC) process that propagates inserts, updates,
-- and (soft) deletes correctly to the data plane
CREATE SEQUENCE rbac.role_binding_last_change_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER TABLE rbac.role_binding_last_change_id_seq OWNER TO cc_rbac;

-- Function to enforce that last_change_id and modified are updated
CREATE OR REPLACE FUNCTION rbac.update_role_binding() RETURNS TRIGGER AS $body$
BEGIN
    NEW.last_change_id = nextval('rbac."role_binding_last_change_id_seq"');
    NEW.modified = NOW();
    RETURN NEW;
END $body$ LANGUAGE plpgsql;

-- Trigger to enforce that last_change_id and modified are updated
CREATE TRIGGER rbac_update_role_binding_trigger
 BEFORE INSERT OR UPDATE ON rbac.role_binding
 FOR EACH ROW
 -- there are currently no other triggers on this table, but this depth check
 -- prevents recursive loops if some are later added.
 WHEN (pg_trigger_depth() < 1)
 EXECUTE PROCEDURE rbac.update_role_binding();

INSERT INTO rbac.role_binding (id, user_id, role_name, created_by, modified_by) VALUES ('rb-000000', 'flowserviceadmin', 'CCloudRoleBindingAdmin', 'seed', 'seed');
